@@ -33,6 +33,7 @@
 #include "virt-viewer-session-spice.h"
 #include "virt-viewer-display-spice.h"
 #include "virt-viewer-auth.h"
+#include "virt-glib-compat.h"
 
 #if !GLIB_CHECK_VERSION(2, 26, 0)
 #include "gbinding.h"
@@ -325,6 +326,7 @@ virt_viewer_session_spice_main_channel_event(SpiceChannel *channel G_GNUC_UNUSED
             gboolean openfd;
 
             g_object_set(self->priv->session, "password", password, NULL);
+            g_free(password);
             g_object_get(self->priv->session, "client-sockets", &openfd, NULL);
 
             if (openfd)
@@ -404,12 +406,93 @@ virt_viewer_session_spice_usb_device_selection(VirtViewerSession *session,
 }
 
 static void
-agent_connected_changed(SpiceChannel *cmain,
+agent_connected_changed(SpiceChannel *cmain G_GNUC_UNUSED,
                         GParamSpec *pspec G_GNUC_UNUSED,
                         VirtViewerSessionSpice *self)
+{
+    // this will force refresh of application menu
+    g_signal_emit_by_name(self, "session-display-updated");
+}
+
+static void
+agent_connected_fullscreen_auto_conf(SpiceChannel *cmain,
+                                     GParamSpec *pspec G_GNUC_UNUSED,
+                                     VirtViewerSessionSpice *self)
 {
     if (virt_viewer_session_spice_fullscreen_auto_conf(self))
-        g_signal_handlers_disconnect_by_func(cmain, agent_connected_changed, self);
+        g_signal_handlers_disconnect_by_func(cmain, agent_connected_fullscreen_auto_conf, self);
+}
+
+static void
+destroy_display(gpointer data)
+{
+    VirtViewerDisplay *display = VIRT_VIEWER_DISPLAY(data);
+    VirtViewerSession *session = virt_viewer_display_get_session(display);
+
+    DEBUG_LOG("Destroying spice display %p", display);
+    virt_viewer_session_remove_display(session, display);
+    g_object_unref(display);
+}
+
+static void
+virt_viewer_session_spice_display_monitors(SpiceChannel *channel,
+                                           GParamSpec *pspec G_GNUC_UNUSED,
+                                           VirtViewerSessionSpice *self)
+{
+    GArray *monitors = NULL;
+    GPtrArray *displays = NULL;
+    GtkWidget *display;
+    guint i, monitors_max;
+
+    g_object_get(channel,
+                 "monitors", &monitors,
+                 "monitors-max", &monitors_max,
+                 NULL);
+    g_return_if_fail(monitors != NULL);
+    g_return_if_fail(monitors->len <= monitors_max);
+
+    displays = g_object_get_data(G_OBJECT(channel), "virt-viewer-displays");
+    if (displays == NULL) {
+        displays = g_ptr_array_new();
+        g_ptr_array_set_free_func(displays, destroy_display);
+        g_object_set_data_full(G_OBJECT(channel), "virt-viewer-displays",
+                               displays, (GDestroyNotify)g_ptr_array_unref);
+    }
+
+    g_ptr_array_set_size(displays, monitors_max);
+
+    for (i = 0; i < monitors_max; i++) {
+        display = g_ptr_array_index(displays, i);
+        if (display == NULL) {
+            display = virt_viewer_display_spice_new(self, channel, i);
+            DEBUG_LOG("creating spice display (#:%d)", i);
+            g_ptr_array_index(displays, i) = g_object_ref(display);
+        }
+
+        g_object_freeze_notify(G_OBJECT(display));
+        virt_viewer_display_set_enabled(VIRT_VIEWER_DISPLAY(display), FALSE);
+        virt_viewer_session_add_display(VIRT_VIEWER_SESSION(self),
+                                        VIRT_VIEWER_DISPLAY(display));
+    }
+
+    for (i = 0; i < monitors->len; i++) {
+        SpiceDisplayMonitorConfig *monitor = &g_array_index(monitors, SpiceDisplayMonitorConfig, i);
+        display = g_ptr_array_index(displays, monitor->id);
+        g_return_if_fail(display != NULL);
+
+        if (monitor->width == 0 || monitor->width == 0)
+            continue;
+
+        virt_viewer_display_set_enabled(VIRT_VIEWER_DISPLAY(display), TRUE);
+        virt_viewer_display_set_desktop_size(VIRT_VIEWER_DISPLAY(display),
+                                             monitor->width, monitor->height);
+    }
+
+    for (i = 0; i < monitors_max; i++)
+        g_object_thaw_notify(g_ptr_array_index(displays, i));
+
+    g_clear_pointer(&monitors, g_array_unref);
+
 }
 
 static void
@@ -438,24 +521,20 @@ virt_viewer_session_spice_channel_new(SpiceSession *s,
                          G_CALLBACK(virt_viewer_session_spice_main_channel_event), self);
         self->priv->main_channel = SPICE_MAIN_CHANNEL(channel);
 
-        g_signal_connect(channel, "notify::agent-connected", G_CALLBACK(agent_connected_changed),  self);
-        agent_connected_changed(channel, NULL, self);
+        g_signal_connect(channel, "notify::agent-connected", G_CALLBACK(agent_connected_changed), self);
+        g_signal_connect(channel, "notify::agent-connected", G_CALLBACK(agent_connected_fullscreen_auto_conf), self);
+        agent_connected_fullscreen_auto_conf(channel, NULL, self);
+
+        g_signal_emit_by_name(session, "session-connected");
     }
 
     if (SPICE_IS_DISPLAY_CHANNEL(channel)) {
-        GtkWidget *display;
-
-        g_signal_emit_by_name(session, "session-connected");
-
-        DEBUG_LOG("new display channel (#%d)", id);
-        display = virt_viewer_display_spice_new(self,
-                                                channel,
-                                                spice_display_new(s, id));
-
-        virt_viewer_session_add_display(VIRT_VIEWER_SESSION(session),
-                                        VIRT_VIEWER_DISPLAY(display));
-
         g_signal_emit_by_name(session, "session-initialized");
+
+        g_signal_connect(channel, "notify::monitors",
+                         G_CALLBACK(virt_viewer_session_spice_display_monitors), self);
+
+        spice_channel_connect(channel);
     }
 
     if (SPICE_IS_INPUTS_CHANNEL(channel)) {
@@ -500,6 +579,10 @@ virt_viewer_session_spice_fullscreen_auto_conf(VirtViewerSessionSpice *self)
 
     DEBUG_LOG("Performing full screen auto-conf, %d host monitors",
               gdk_screen_get_n_monitors(screen));
+    g_object_set(G_OBJECT(cmain),
+                 "disable-display-position", FALSE,
+                 "disable-display-align", TRUE,
+                 NULL);
     spice_main_set_display_enabled(cmain, -1, FALSE);
     for (i = 0; i < gdk_screen_get_n_monitors(screen); i++) {
         gdk_screen_get_monitor_geometry(screen, i, &dest);
@@ -533,7 +616,8 @@ virt_viewer_session_spice_channel_destroy(G_GNUC_UNUSED SpiceSession *s,
     }
 
     if (SPICE_IS_DISPLAY_CHANNEL(channel)) {
-        DEBUG_LOG("zap session channel (#%d)", id);
+        VirtViewerDisplay *display = g_object_get_data(G_OBJECT(channel), "virt-viewer-display");
+        DEBUG_LOG("zap display channel (#%d, %p)", id, display);
     }
 
     if (SPICE_IS_PLAYBACK_CHANNEL(channel) && self->priv->audio) {
